==> ldc_1.20.1.orig.tar.gz: ldc-1.20.1-src/gen/attributes.cpp <==
//===-- attributes.cpp ----------------------------------------------------===//
//
//                         LDC – the LLVM D compiler
//
// This file is distributed under the BSD-style LDC license. See the LICENSE
// file for details.
//
//===----------------------------------------------------------------------===//

#include "gen/attributes.h"
#include "gen/irstate.h"

AttrSet::AttrSet(const AttrSet &base, unsigned index, LLAttribute attribute)
    : set(base.set.addAttribute(gIR->context(), index, attribute)) {}

AttrSet
AttrSet::extractFunctionAndReturnAttributes(const llvm::Function *function) {
  auto old = function->getAttributes();
#if LDC_LLVM_VER >= 500
  return {LLAttributeSet::get(gIR->context(), old.getFnAttributes(),
                              old.getRetAttributes(), {})};
#else
  llvm::AttributeSet existingAttrs[] = {old.getFnAttributes(),
                                        old.getRetAttributes()};
  return {LLAttributeSet::get(gIR->context(), existingAttrs)};
#endif
}

AttrSet &AttrSet::add(unsigned index, const llvm::AttrBuilder &builder) {
  if (builder.hasAttributes()) {
#if LDC_LLVM_VER >= 500
    set = set.addAttributes(gIR->context(), index, builder);
#else
    auto as = LLAttributeSet::get(gIR->context(), index, builder);
    set = set.addAttributes(gIR->context(), index, as);
#endif
  }
  return *this;
}

AttrSet &AttrSet::merge(const AttrSet &other) {
  auto &os = other.set;
#if LDC_LLVM_VER >= 500
  set = LLAttributeSet::get(gIR->context(), {set,os});
#else
  for (unsigned i = 0; i < os.getNumSlots(); ++i) {
    unsigned index = os.getSlotIndex(i);
    set = set.addAttributes(gIR->context(), index, os.getSlotAttributes(i));
  }
#endif
  return *this;
}

==> ldc_1.28.0.orig.tar.gz: ldc-1.28.0-src/gen/attributes.cpp <==
//===-- attributes.cpp ----------------------------------------------------===//
//
//                         LDC – the LLVM D compiler
//
// This file is distributed under the BSD-style LDC license. See the LICENSE
// file for details.
//
//===----------------------------------------------------------------------===//

#include "gen/attributes.h"
#include "gen/irstate.h"

AttrSet::AttrSet(const AttrSet &base, unsigned index, LLAttribute attribute)
#if LDC_LLVM_VER >= 1400
    : set(base.set.addAttributeAtIndex(gIR->context(), index, attribute)) {}
#else
    : set(base.set.addAttribute(gIR->context(), index, attribute)) {}
#endif

AttrSet
AttrSet::extractFunctionAndReturnAttributes(const llvm::Function *function) {
  auto old = function->getAttributes();
  return {LLAttributeList::get(gIR->context(),
#if LDC_LLVM_VER >= 1400
                               old.getFnAttrs(),
                               old.getRetAttrs(),
#else
                               old.getFnAttributes(),
                               old.getRetAttributes(),
#endif
                               {})};
}

AttrSet &AttrSet::add(unsigned index, const llvm::AttrBuilder &builder) {
  if (builder.hasAttributes()) {
#if LDC_LLVM_VER >= 1400
    set = set.addAttributesAtIndex(gIR->context(), index, builder);
#else
    set = set.addAttributes(gIR->context(), index, builder);
#endif
  }
  return *this;
}

AttrSet &AttrSet::merge(const AttrSet &other) {
  auto &os = other.set;
  set = LLAttributeList::get(gIR->context(), {set, os});
  return *this;
}
